#ifndef _BSShell_interface_
#define _BSShell_interface_

#include <stdio.h>
#include "Parser.hh"
#include "buildin/Builtin.hh"

/**
 * BSSShell ist ein Kommandozeileninterpreter. 
 * Nach seiner Initialisierung startet er eine Schleife, die erst durch den
 * Aufruf von <code>exit</code> oder einen Fehler beendet wird. In dieser Schleife
 * wird eine Kommandozeile eingelesen und analysiert, um danach eine dem 
 * eingelesenen Kommando enstsprechende Methode auszufhren.
 *
 * @author Arbeitsgruppe EOS
 */
class BSShell : public Parser {
private:
    /**
     * bestimmt den Prompt der Kommandozeile des Parsers
     */
    const char *prompt;
    
    /**
     * verwaltet die internen Kommandos des Parsers
     */
    Builtin internal_cmd;
     
    /**
     * Liest eine Kommandozeile ein. 
     *
     * @param buffer Pointer auf den Puffer, in den die Kommandozeile eingelesen wird
     * @param size Gre des bergebenen Puffers
     * @return Liefert die Anzahl der eingelesenen Zeichen oder einen Fehlercode.
     */
    int readline(char *buffer, const int size);
    
    /**
     * Fhrt die eingelesenen Kommandos im Interpreter aus.
     * Die Methode <code>execute()</code> fhrt ausschlielich interne Kommandos
     * aus, das Ausfhren von Kommandos, die nicht im Interpreter implementiert
     * sind, ist nicht vorgesehen.
     *
     * @return Liefert true, wenn das Kommando <code>exit</code> aufgerufen
     *         wurde und der Interpreter beendet werden soll.
     */
    bool execute();
public:
    /**
     * Erzeugt eine Instanze des Interpreters. 
     *
     * @param prompt Prompt der Kommandozeile des Parsers
     */
    BSShell(const char *prompt = "bssh> ");
    ~BSShell();
    
    /**
     * Startet die Ausfhrungsschleife des Interpreters.
     * Die Bearbeitung erfolgt dabei nach dem folgenden Schema: Einlesen und 
     * Analysieren einer Kommandozeile und danach Ausfhren der eingelesenen
     * Kommandos. Die Schleife wird solange abgearbeitet, bis der Interpreter
     * durch den Aufruf von <code>exit</code> oder einen Fehler beendet wird.
     *
     * @return Liefert true, wenn der Interpreter mit dem Aufruf von 
     *         <code>exit</code> benendet wurde.
     */
    bool loop();
};

#endif // _BSShell_interface_
